module StochasticProcess
  class Base
    DEFAULT_START_TIME = 0.0
    DEFAULT_END_TIME = 100
    DEFAULT_EVALUATIONS = 1000

    attr_accessor :evaluations

    def initialize(initial_position: 0.0,
      start_time: DEFAULT_START_TIME,
      end_time: DEFAULT_END_TIME,
      evaluations: DEFAULT_EVALUATIONS,
      path_increment: method(:default_path_increment))
      @initial_position = initial_position
      @start_time = start_time
      @end_time = end_time
      @evaluations = evaluations
      @path_increment = path_increment
    end

    def default_path_increment
      return 0
    end

    def interval_length
      (@end_time - @start_time).abs.to_f
    end

    def mesh
       interval_length / @evaluations
    end

    def mesh=(m)
      if m <= 0
        raise("Only positive numbers allowed!")
      end
      @evaluations = interval_length / m.to_f
    end

    def graph
      a = []
      m = mesh()
      a.push({x: @start_time, y: @initial_position})
      @evaluations.times do |i| # iterates from 0 to @evaluations - 1
        a.push({x: @start_time + (i + 1) * m, y: a[i][:y] + @path_increment.call})
      end
      return a
    end

  end
end
