module StochasticProcess

  class BrownianMotion < Base

    def brownian_path_increment
      return gaussian(0, 0.1)
    end

    def initialize(initial_position: 0.0,
      start_time: DEFAULT_START_TIME,
      end_time: DEFAULT_END_TIME,
      evaluations: DEFAULT_EVALUATIONS)
      super(initial_position: initial_position, start_time: start_time,
        end_time: end_time, evaluations: evaluations,
        path_increment: method(:brownian_path_increment))
    end

    private

    def gaussian(mean, stddev)
      theta = 2 * Math::PI * rand
      rho = Math.sqrt(-2 * Math.log(1 - rand))
      scale = stddev * rho
      x = mean + scale * Math.cos(theta)
      #y = mean + scale * Math.sin(theta)
      return x
    end
  end
end